@@ -27,6 +27,7 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 #define dwarf_h
 
 #include <libunwind.h>
+#include <stdatomic.h>
 
 struct dwarf_cursor;    /* forward-declaration */
 struct elf_dyn_info;
@@ -45,6 +46,9 @@ struct elf_dyn_info;
   #else
     #error Could not find <link.h>
   #endif
+  #if defined(__ANDROID__) && defined(__arm__) && __ANDROID_API__ < 21
+    int dl_iterate_phdr(int (*)(struct dl_phdr_info *, size_t, void *), void *);
+  #endif
 #endif
 
 #include <pthread.h>
@@ -347,7 +351,7 @@ struct dwarf_rs_cache
     /* hash table that maps instruction pointer to rs index: */
     unsigned short *hash;
 
-    uint32_t generation;        /* generation number */
+    _Atomic uint32_t generation;        /* generation number */
 
     /* rs cache: */
     dwarf_reg_state_t *buckets;
@@ -366,6 +370,8 @@ struct unw_debug_frame_list
     /* The start (inclusive) and end (exclusive) of the described region.  */
     unw_word_t start;
     unw_word_t end;
+    /* ELF load offset */
+    unw_word_t load_offset;
     /* The debug frame itself.  */
     char *debug_frame;
     size_t debug_frame_size;
