@@ -31,6 +31,7 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 
 #include <stdlib.h>
 #include <libunwind.h>
+#include <stdatomic.h>
 
 # include "elf64.h"
 #include "mempool.h"
@@ -50,11 +51,7 @@ struct unw_addr_space
   unsigned int addr_size;
 
   unw_caching_policy_t caching_policy;
-#ifdef HAVE_ATOMIC_OPS_H
-  AO_t cache_generation;
-#else
-  uint32_t cache_generation;
-#endif
+  _Atomic uint32_t cache_generation;
   unw_word_t dyn_generation;          /* see dyn-common.h */
   unw_word_t dyn_info_list_addr;      /* (cached) dyn_info_list_addr */
   struct dwarf_rs_cache global_cache;
@@ -235,7 +232,7 @@ dwarf_put (struct dwarf_cursor *c, dwarf_loc_t loc, unw_word_t val)
 #define tdep_get_as_arg(c)              ((c)->dwarf.as_arg)
 #define tdep_get_ip(c)                  ((c)->dwarf.ip)
 
-extern int tdep_init_done;
+extern atomic_bool tdep_init_done;
 
 extern void tdep_init (void);
 extern int tdep_search_unwind_table (unw_addr_space_t as,
