@@ -31,7 +31,9 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 /* Target-dependent definitions that are internal to libunwind but need
    to be shared with target-independent code.  */
 
+#include <stdint.h>
 #include <stdlib.h>
+#include <stdatomic.h>
 #include <libunwind.h>
 
 #include "elf64.h"
@@ -40,6 +42,7 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 
 typedef enum
   {
+    UNW_X86_64_FRAME_ALIGNED = -3,       /* frame stack pointer aligned */
     UNW_X86_64_FRAME_STANDARD = -2,     /* regular rbp, rsp +/- offset */
     UNW_X86_64_FRAME_SIGRETURN = -1,    /* special sigreturn frame */
     UNW_X86_64_FRAME_OTHER = 0,         /* not cacheable (special or unrecognised) */
@@ -50,10 +53,10 @@ unw_tdep_frame_type_t;
 typedef struct
   {
     uint64_t virtual_address;
-    int64_t frame_type     : 2;  /* unw_tdep_frame_type_t classification */
+    int64_t frame_type     : 3;  /* unw_tdep_frame_type_t classification */
     int64_t last_frame     : 1;  /* non-zero if last frame in chain */
     int64_t cfa_reg_rsp    : 1;  /* cfa dwarf base register is rsp vs. rbp */
-    int64_t cfa_reg_offset : 30; /* cfa is at this offset from base register value */
+    int64_t cfa_reg_offset : 29; /* cfa is at this offset from base register value */
     int64_t rbp_cfa_offset : 15; /* rbp saved at this offset from cfa (-1 = not saved) */
     int64_t rsp_cfa_offset : 15; /* rsp saved at this offset from cfa (-1 = not saved) */
   }
@@ -63,11 +66,7 @@ struct unw_addr_space
   {
     struct unw_accessors acc;
     unw_caching_policy_t caching_policy;
-#ifdef HAVE_ATOMIC_OPS_H
-    AO_t cache_generation;
-#else
-    uint32_t cache_generation;
-#endif
+    _Atomic uint32_t cache_generation;
     unw_word_t dyn_generation;          /* see dyn-common.h */
     unw_word_t dyn_info_list_addr;      /* (cached) dyn_info_list_addr */
     struct dwarf_rs_cache global_cache;
@@ -88,18 +87,48 @@ struct cursor
         X86_64_SCF_LINUX_RT_SIGFRAME,   /* Linux ucontext_t */
         X86_64_SCF_FREEBSD_SIGFRAME,    /* FreeBSD signal frame */
         X86_64_SCF_FREEBSD_SYSCALL,     /* FreeBSD syscall */
+        X86_64_SCF_SOLARIS_SIGFRAME,    /* illumos/Solaris signal frame */
       }
     sigcontext_format;
     unw_word_t sigcontext_addr;
-    int validate;
-    ucontext_t *uc;
   };
 
+#define AS_ARG_UCONTEXT_MASK ~0x1UL
+#define AS_ARG_VALIDATE_MASK 0x1UL
+
+#define AS_ARG_GET_UC_PTR(arg) \
+  ((ucontext_t *) ((uintptr_t) arg & AS_ARG_UCONTEXT_MASK))
+#define AS_ARG_GET_VALIDATE(arg) \
+  ((int) ((uintptr_t) arg & AS_ARG_VALIDATE_MASK))
+
 static inline ucontext_t *
 dwarf_get_uc(const struct dwarf_cursor *cursor)
 {
-  const struct cursor *c = (struct cursor *) cursor->as_arg;
-  return c->uc;
+  assert(cursor->as == unw_local_addr_space);
+  return AS_ARG_GET_UC_PTR(cursor->as_arg);
+}
+
+static inline int
+dwarf_get_validate(const struct dwarf_cursor *cursor)
+{
+  assert(cursor->as == unw_local_addr_space);
+  return AS_ARG_GET_VALIDATE(cursor->as_arg);
+}
+
+static inline void
+dwarf_set_validate(const struct dwarf_cursor *cursor, const int validate)
+{
+  assert(cursor->as == unw_local_addr_space);
+  uintptr_t *packed_args = (uintptr_t *) &cursor->as_arg;
+  *packed_args |= (AS_ARG_VALIDATE_MASK & validate);
+}
+
+static inline void *
+dwarf_build_as_arg(const ucontext_t *uc, const int validate) {
+  uintptr_t packed_args = (uintptr_t) uc;
+  assert((packed_args & AS_ARG_VALIDATE_MASK) == 0);
+  packed_args |= (AS_ARG_VALIDATE_MASK & validate);
+  return (void *) packed_args;
 }
 
 #define DWARF_GET_LOC(l)        ((l).val)
@@ -128,8 +157,14 @@ dwarf_get_uc(const struct dwarf_cursor *cursor)
 #else /* !UNW_LOCAL_ONLY */
 
 # define DWARF_NULL_LOC         DWARF_LOC (0, 0)
-# define DWARF_IS_NULL_LOC(l)                                           \
-                ({ dwarf_loc_t _l = (l); _l.val == 0 && _l.type == 0; })
+
+static inline int
+dwarf_is_null_loc(dwarf_loc_t l)
+{
+  return l.val == 0 && l.type == 0;
+}
+
+# define DWARF_IS_NULL_LOC(l)   dwarf_is_null_loc(l)
 # define DWARF_REG_LOC(c,r)     DWARF_LOC((r), DWARF_LOC_TYPE_REG)
 # define DWARF_FPREG_LOC(c,r)   DWARF_LOC((r), (DWARF_LOC_TYPE_REG      \
                                                 | DWARF_LOC_TYPE_FP))
@@ -196,6 +231,7 @@ dwarf_put (struct dwarf_cursor *c, dwarf_loc_t loc, unw_word_t val)
 #define tdep_search_unwind_table        dwarf_search_unwind_table
 #define tdep_find_unwind_table          dwarf_find_unwind_table
 #define tdep_get_elf_image              UNW_ARCH_OBJ(get_elf_image)
+#define tdep_get_exe_image_path         UNW_ARCH_OBJ(get_exe_image_path)
 #define tdep_access_reg                 UNW_OBJ(access_reg)
 #define tdep_access_fpreg               UNW_OBJ(access_fpreg)
 #if __linux__
@@ -204,8 +240,8 @@ dwarf_put (struct dwarf_cursor *c, dwarf_loc_t loc, unw_word_t val)
 # define tdep_reuse_frame               UNW_OBJ(reuse_frame)
 #else
 # define tdep_fetch_frame(c,ip,n)       do {} while(0)
-# define tdep_cache_frame(c,rs)         do {} while(0)
-# define tdep_reuse_frame(c,rs)         do {} while(0)
+# define tdep_cache_frame(c)            0
+# define tdep_reuse_frame(c,frame)      do {} while(0)
 #endif
 #define tdep_stash_frame                UNW_OBJ(stash_frame)
 #define tdep_trace                      UNW_OBJ(tdep_trace)
@@ -230,7 +266,7 @@ dwarf_put (struct dwarf_cursor *c, dwarf_loc_t loc, unw_word_t val)
 #define tdep_get_ip(c)                  ((c)->dwarf.ip)
 #define tdep_big_endian(as)             0
 
-extern int tdep_init_done;
+extern atomic_bool tdep_init_done;
 
 extern void tdep_init (void);
 extern void tdep_init_mem_validate (void);
@@ -241,6 +277,7 @@ extern void *x86_64_r_uc_addr (ucontext_t *uc, int reg);
 extern int tdep_get_elf_image (struct elf_image *ei, pid_t pid, unw_word_t ip,
                                unsigned long *segbase, unsigned long *mapoff,
                                char *path, size_t pathlen);
+extern void tdep_get_exe_image_path (char *path);
 extern int tdep_access_reg (struct cursor *c, unw_regnum_t reg,
                             unw_word_t *valp, int write);
 extern int tdep_access_fpreg (struct cursor *c, unw_regnum_t reg,
@@ -248,10 +285,9 @@ extern int tdep_access_fpreg (struct cursor *c, unw_regnum_t reg,
 #if __linux__
 extern void tdep_fetch_frame (struct dwarf_cursor *c, unw_word_t ip,
                               int need_unwind_info);
-extern void tdep_cache_frame (struct dwarf_cursor *c,
-                              struct dwarf_reg_state *rs);
+extern int tdep_cache_frame (struct dwarf_cursor *c);
 extern void tdep_reuse_frame (struct dwarf_cursor *c,
-                              struct dwarf_reg_state *rs);
+                              int frame);
 extern void tdep_stash_frame (struct dwarf_cursor *c,
                               struct dwarf_reg_state *rs);
 #endif
