@@ -70,7 +70,7 @@ aarch64_handle_signal_frame (unw_cursor_t *cursor)
   c->sigcontext_sp = c->dwarf.cfa;
   c->sigcontext_pc = c->dwarf.ip;
 
-  if (ret)
+  if (ret > 0)
     {
       c->sigcontext_format = AARCH64_SCF_LINUX_RT_SIGFRAME;
       sc_addr = sp_addr + sizeof (siginfo_t) + LINUX_UC_MCONTEXT_OFF;
@@ -134,14 +134,30 @@ int
 unw_step (unw_cursor_t *cursor)
 {
   struct cursor *c = (struct cursor *) cursor;
+  int validate = c->validate;
   int ret;
 
   Debug (1, "(cursor=%p, ip=0x%016lx, cfa=0x%016lx))\n",
          c, c->dwarf.ip, c->dwarf.cfa);
 
+  /* Validate all addresses before dereferencing. */
+  c->validate = 1;
+
   /* Check if this is a signal frame. */
-  if (unw_is_signal_frame (cursor) > 0)
+  ret = unw_is_signal_frame (cursor);
+  if (ret > 0)
     return aarch64_handle_signal_frame (cursor);
+  else if (unlikely (ret < 0))
+    {
+      /* IP points to non-mapped memory. */
+      /* This is probably SIGBUS. */
+      /* Try to load LR in IP to recover. */
+      Debug(1, "Invalid address found in the call stack: 0x%lx\n", c->dwarf.ip);
+      dwarf_get (&c->dwarf, c->dwarf.loc[UNW_AARCH64_X30], &c->dwarf.ip);
+    }
+
+  /* Restore default memory validation state */
+  c->validate = validate;
 
   ret = dwarf_step (&c->dwarf);
   Debug(1, "dwarf_step()=%d\n", ret);
