@@ -33,8 +33,14 @@ unw_create_addr_space (unw_accessors_t *a, int byte_order)
 #ifdef UNW_LOCAL_ONLY
   return NULL;
 #else
-  unw_addr_space_t as = malloc (sizeof (*as));
+  /*
+   * Tilegx supports only big or little-endian, not weird stuff like
+   * PDP_ENDIAN.
+   */
+  if (byte_order != 0 && byte_order_is_valid(byte_order) == 0)
+    return NULL;
 
+  unw_addr_space_t as = malloc (sizeof (*as));
   if (!as)
     return NULL;
 
@@ -42,20 +48,11 @@ unw_create_addr_space (unw_accessors_t *a, int byte_order)
 
   as->acc = *a;
 
-  /*
-   * Tilegx supports only big or little-endian, not weird stuff like
-   * PDP_ENDIAN.
-   */
-  if (byte_order != 0
-      && byte_order != __LITTLE_ENDIAN
-      && byte_order != __BIG_ENDIAN)
-    return NULL;
-
   if (byte_order == 0)
     /* use host default: */
-    as->big_endian = (__BYTE_ORDER == __BIG_ENDIAN);
+    as->big_endian = target_is_big_endian();
   else
-    as->big_endian = (byte_order == __BIG_ENDIAN);
+    as->big_endian = (byte_order == UNW_BIG_ENDIAN);
 
   as->abi = UNW_TILEGX_ABI_N64;
   as->addr_size = 8;
