@@ -178,7 +178,7 @@ main (int argc, char **argv)
 
   if (argc == 1)
     {
-      static char *args[] = { "self", "/bin/ls", "/usr", NULL };
+      static char *args[] = { "self", "ls", "/", NULL };
 
       /* automated test case */
       argv = args;
@@ -233,7 +233,11 @@ main (int argc, char **argv)
         fprintf(stderr, "Need to specify a command line for the child\n");
         exit (-1);
       }
+#ifdef __FreeBSD__
       execve (argv[optind], argv + optind, environ);
+#else
+      execvpe (argv[optind], argv + optind, environ);
+#endif
       _exit (-1);
     }
   atexit (target_pid_kill);
